import string
import gtk
import gobject
import rhpl.monitor

TRUE = 1
FALSE = 0

class MonitorDialog:
    def __init__(self, xml, monitor, probed_id, hardware_state, monitor_name=None):
        self.xml = xml
        self.monitor = monitor
        self.probed_id = probed_id
        self.hardware_state = hardware_state
        self.monitor_name = monitor_name

        xml.get_widget("monitor_icon").set_from_file("/usr/share/redhat-config-xfree86/pixmaps/monitor.png")

        self.probed_path = None

        self.changing_selection = 0
        
        dialog = self.xml.get_widget("monitor_dialog")
        dialog.set_transient_for(self.xml.get_widget("display_dialog"))
        dialog.set_modal(TRUE)

        self.monitor_store = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING,
                                      gobject.TYPE_STRING, gobject.TYPE_STRING)

        self.tree_view = self.xml.get_widget("monitor_tree")
        self.tree_view.set_model(self.monitor_store)
        
        selection = self.tree_view.get_selection()
        selection.connect("changed", self.selection_changed)

        self.col = gtk.TreeViewColumn(None, gtk.CellRendererText(), text=0)
        self.tree_view.append_column(self.col)

        button = self.xml.get_widget("monitor_probe_button")
        button.connect("clicked", self.probe_monitor)
        button.set_sensitive(self.probed_id != "Unprobed Monitor")

    def selection_changed(self, selection):
        if self.changing_selection:
            return
        (store, iter) = selection.get_selected()
        if iter == None:
            return

        if store.get_value(iter, 1) != None:
            model = store.get_value(iter, 0)
            vsync = store.get_value(iter, 2)
            hsync = store.get_value(iter, 3)

    def probe_monitor(self, widget):
        if self.probed_path:
            self.goto_path(self.probed_path)
            
    def goto_path(self, path):
        self.tree_view = self.xml.get_widget("monitor_tree")
        self.tree_view.expand_row(path[:-1], TRUE)
        self.changing_selection = 1
        selection = self.tree_view.get_selection()
        selection.unselect_all()
        iter = self.monitor_store.get_iter(path)
        selection.select_iter(iter)
        self.changing_selection = 0
        self.tree_view.set_cursor(path, self.col, FALSE)
        self.tree_view.scroll_to_cell(path, self.col, TRUE, 0.5, 0.0)
        self.tree_view.grab_focus ()

    def hydrate_primary_monitor(self):
        db = self.monitor.readMonitorsDB()
        l = db.keys()
        l.sort()

        #put Generic LCD and Generic CRT at the front of the list
        tmp = l[l.index("Generic LCD Display")]
        l.remove(l[l.index("Generic LCD Display")])
        l = [tmp] + l

        tmp = l[l.index("Generic CRT Display")]
        l.remove(l[l.index("Generic CRT Display")])
        l = [tmp] + l

        for maker in l:
            parent = self.monitor_store.append(None)
            self.monitor_store.set_value(parent, 0, maker)
            for mon in db[maker]:
                iter = self.monitor_store.append(parent)
                self.monitor_store.set_value(iter, 0, mon[0])
                self.monitor_store.set_value(iter, 1, mon[1])
                self.monitor_store.set_value(iter, 2, mon[2])
                self.monitor_store.set_value(iter, 3, mon[3])

                if self.monitor_name == "Unknown monitor":
                    #The name in the file is "Unknown monitor", so let's try to probe
                    if self.probed_id and string.upper(mon[1]) == string.upper(self.probed_id):
                        self.probed_path = self.monitor_store.get_path(iter) 
                    elif self.hardware_state.get_monitor_name() == mon[0]:
                        #We couldn't probe the monitor (no self.probed_id), but maybe we have the name
                        #if the user has previously selected it in the list
                        self.probed_path = self.monitor_store.get_path(iter)
                else:
                    #We've found a name in the file, so use that
                    if self.monitor_name == mon[0]:
                        self.probed_path = self.monitor_store.get_path(iter)
                    

        if self.probed_path:
            self.goto_path(self.probed_path)

    def dehydrate_primary_monitor(self, state):
        (store, iter) = self.tree_view.get_selection().get_selected()
        if iter == None:
            return

        if store.get_value(iter, 1) != None:
            model = store.get_value(iter, 0)
            vsync = store.get_value(iter, 2)
            hsync = store.get_value(iter, 3)

        state.set_monitor_name(model)
        state.set_hsync(hsync)
        state.set_vsync(vsync)

    def hydrate_second_monitor(self):
        print "in hydrate_second"
        db = self.monitor.readMonitorsDB()
        l = db.keys()
        l.sort()

        #put Generic LCD and Generic CRT at the front of the list
        tmp = l[l.index("Generic LCD Display")]
        l.remove(l[l.index("Generic LCD Display")])
        l = [tmp] + l

        tmp = l[l.index("Generic CRT Display")]
        l.remove(l[l.index("Generic CRT Display")])
        l = [tmp] + l

        for maker in l:
            parent = self.monitor_store.append(None)
            self.monitor_store.set_value(parent, 0, maker)
            for mon in db[maker]:
                iter = self.monitor_store.append(parent)
                self.monitor_store.set_value(iter, 0, mon[0])
                self.monitor_store.set_value(iter, 1, mon[1])
                self.monitor_store.set_value(iter, 2, mon[2])
                self.monitor_store.set_value(iter, 3, mon[3])
                
                if self.monitor_name == mon[0]:
                    self.probed_path = self.monitor_store.get_path(iter)
                    
        if self.probed_path:
            self.goto_path(self.probed_path)

    def dehydrate_second_monitor(self, state):
        (store, iter) = self.tree_view.get_selection().get_selected()
        if iter == None:
            return

        if store.get_value(iter, 1) != None:
            model = store.get_value(iter, 0)
            vsync = store.get_value(iter, 2)
            hsync = store.get_value(iter, 3)

        return model, vsync, hsync

    def run(self):
        dialog = self.xml.get_widget("monitor_dialog")
        dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        dialog.present()
        while 1:
            res = dialog.run()
            if res != gtk.RESPONSE_OK:
                dialog.hide()
                return FALSE
            
            break
        
        dialog.hide()
        return TRUE
